/*************************************************************************
*  Copyright (C) 2004 by Olivier Galizzi                                 *
*  olivier.galizzi@imag.fr                                               *
*  Copyright (C) 2004 by Janek Kozicki                                   *
*  cosurgi@berlios.de                                                    *
*                                                                        *
*  This program is free software; it is licensed under the terms of the  *
*  GNU General Public License v2 or later. See file LICENSE for details. *
*************************************************************************/

#include"NewtonsForceLaw.hpp"
#include<yade/pkg-common/ParticleParameters.hpp>
#include<yade/pkg-common/RigidBodyParameters.hpp>
#include<yade/core/MetaBody.hpp>

void NewtonsForceLaw::go(const shared_ptr<PhysicalParameters>& b, const Body* bb, MetaBody* rb){
	Vector3r f=rb->bex.getForce(bb->getId());
	ParticleParameters * p = YADE_CAST<ParticleParameters*>(b.get());
	
	// normal behavior of a standalone particle or a clump itself
	if (bb->isStandalone()) p->acceleration=f/p->mass;
	else if (bb->isClump()) {
		// accel for clump reset in Clump::moveMembers, called by ClumpMemberMover engine
		p->acceleration+=f/p->mass;
	}
	else{ // force applied to a clump member is applied to clump itself
		const shared_ptr<Body>& clump(Body::byId(bb->clumpId));
		RigidBodyParameters* clumpRBP=YADE_CAST<RigidBodyParameters*>(clump->physicalParameters.get());
		// accels reset by Clump::moveMembers in last iteration
		clumpRBP->acceleration+=f/clumpRBP->mass;
		clumpRBP->angularAcceleration+=diagDiv((b->se3.position-clumpRBP->se3.position).Cross(f),clumpRBP->inertia); //acceleration from torque generated by the force WRT particle centroid on the clump centroid
	}
}


YADE_PLUGIN((NewtonsForceLaw));
